import { CamelCase, PascalCase } from 'type-fest';
import * as _ from 'lodash';

/** 将类型 T 的键递归的转为小驼峰 */
export type CamelCasedProps<T> = T extends Array<infer U>
  ? Array<CamelCasedProps<U>>
  : {
      [K in keyof T as CamelCase<K>]: CamelCasedProps<T[K]>;
    };

/** 将类型 T 的键递归的转为大驼峰 */
export type PascalCasedProps<T> = T extends Array<infer U>
  ? Array<PascalCasedProps<U>>
  : {
      [K in keyof T as PascalCase<K>]: PascalCasedProps<T[K]>;
    };

/**
 * 将字符串转为大驼峰
 *
 * @param str 要转换的字符串
 * @returns 转换完的字符串
 */
export function pascalCase<T extends string>(str: T): PascalCase<T> {
  return _.startCase(_.camelCase(str)).replace(/ /g, '') as PascalCase<T>;
}

/**
 * 将 object 深层的转为大驼峰
 *
 * @param obj 要转换的对象
 * @returns 转换的结果
 */
export function pascalCaseProperty<T extends Record<string, any>>(obj: T): PascalCasedProps<T> {
  let res: Record<string, any> = {};
  if (_.isObject(obj)) {
    res = {} as T;
    Object.keys(obj).forEach((key: string) => {
      const val = obj[key];
      const k = pascalCase(key);
      res[k] = _.isObject(val) || _.isArray(val) ? pascalCaseProperty(val) : val;
    });
  }
  if (_.isArray(obj as any)) {
    res = [];
    obj.forEach((item: any) => {
      res.push(_.isObject(item) || _.isArray(item) ? pascalCaseProperty(item) : item);
    });
  }
  return res as PascalCasedProps<T>;
}

/**
 * 深层的移除对象中的 undefined（只支持 plain object）
 *
 * @param obj 要移除 undefined 的对象
 * @returns 移除完的结果
 */
export function removeUndefined<T>(obj: T): T {
  return JSON.parse(JSON.stringify(obj));
}

/**
 * 深拷贝对象（只支持 plain object）
 *
 * @param obj 要深拷贝的对象
 * @returns 拷贝完成的结果
 */
export function deepClone<T>(obj: T): T {
  return JSON.parse(JSON.stringify(obj));
}
